%{
 _   _            _
| | | |          | |
| |_| | ___  __ _| |_   _ __ ___  ___ _____   _____ _ __ _   _
|  _  |/ _ \/ _` | __| | '__/ _ \/ __/ _ \ \ / / _ \ '__| | | |
| | | |  __/ (_| | |_  | | |  __/ (_| (_) \ V /  __/ |  | |_| |
\_| |_/\___|\__,_|\__| |_|  \___|\___\___/ \_/ \___|_|   \__, |
               _       _                                  __/ |
              (_)     | |                                |___/
 ___  ___ _ __ _ _ __ | |_
/ __|/ __| '__| | '_ \| __|
\__ \ (__| |  | | |_) | |_
|___/\___|_|  |_| .__/ \__|
                | |
                |_|
%}
clear all
clc
total_timer=tic;
fprintf('Please wait.');
%{
Input data from IDA ICE (stored in file with filename "input.xlsx"):
T_ahe:      Temperature after rotating heat exchanger [deg C]
V_ea:       Exhaust air flow [m3/h]
Q_dem:      Heat demand [W]
T_dhw_in:   Temperature of incoming water from DHW tank [deg C]
m_dhw_cold: Mass flow of incoming water from DHW tank [kg/s]
T_sh_in:    Temperature of incoming water from space heating tank [deg C]
m_sh_cold:  Mass flow of incoming water from SH tank [kg/s]
T_oa:       Outdoor air temperature [deg C]
T_ea:       Temperature of exhaust air from zones (before h.ex.) [deg C]

Note! Use ".xlsx" Excel file type. File must be stored in '../input/'
The Matlab code is valid for data with a time step of one hour only.
%}

% Loading parameters, constants and preallocations:
parameters
constants
preallocations

% Importing input data from excel file "input.xlsx":
temp=xlsread('../input/input.xlsx','','A:I','basic');
T_ahe=temp(:,1);
V_ea=temp(:,2).*3.6;
Q_dem=temp(:,3);
T_dhw_in=temp(:,4);
m_dhw_cold=temp(:,5);
T_sh_in=temp(:,6);
m_sh_cold=temp(:,7);
T_oa=temp(:,8);
T_ea=temp(:,9);
n=length(T_ahe);

% Start of initial error checking:
if ~activate_oa && ~activate_ea
    fprintf('No evaporator heat source is chosen. \nAborting script.\n');
    return
end

if n~=length(V_ea) || n~=length(Q_dem) || n~=length(T_dhw_in) || ...
        n~=length(m_dhw_cold) || n~=length(T_sh_in) || ...
        n~=length(m_sh_cold) || n~=length(T_oa) || n~=length(T_ea)
    fprintf(['Error in input.xlsx or in the interpretation of this file.\n' ...
        'Length of coloumns not equal.\n Aborting script.\n']);
    return
elseif n~=8760
    fprintf(['Number of time steps is not equal to 8760.\n'...
        'Aborting script.\n']);
    return
end

for i=1:8760
    % IDA ICE may return negative water mass flows, resulting in invalid
    % T_cw temperatures. The real mass flow should be 0 or close to 0.
    if m_dhw_cold(i)<0
        m_dhw_cold(i)=10^(-6);
    end
    if m_sh_cold(i)<0
        m_sh_cold(i)=10^(-6);
    end
end

%%%%%% End of initial error checking %%%%%%

% Water from DHW and space heating tank are mixed before entering GC:
T_cw=(m_dhw_cold.*T_dhw_in+m_sh_cold.*T_sh_in)./(m_dhw_cold+m_sh_cold);

% Calculating outdoor air fan power
if activate_oa
    W_oa_fan(1:8760,1)=SFP_oa_fan*v_oa/3.6;      %3.6 used to convert units
    Q_heating_oa=W_oa_fan.*eta_oa_fan;
    T_oa_new=T_oa+Q_heating_oa/(hcap_air*v_oa);    
end

% If true, it means an outdoor air to water heat pump
if activate_oa && ~activate_ea
    T_ahe=T_oa_new;
    V_ea(:,1)=v_oa;
    W_ea_fan=W_oa_fan;
    activate_oa=false;
    activate_ea=true;
    oa_only=true;
else
    oa_only=false;
end


%{
Establishing thermodynamical variables and relations:

Temperature [deg C]
   ^                                     * * 2
  / \                                 *   *
   |                               *     *
T  |                             *      *
e  |                          *        *
m  |                       *          *
p  |                    *            *
e  |         *  * *  *              *
r  |       *  _______              *
a  |     3*  /       \            *       ******  = CO2 process line
t  |      * /         \          *
u  |      */           \        *      ___
r  |      /             \      *      /   \   = Saturation curve CO2
e  |     /*              \    *      /     \
   |    / *               \  *
   |   /  *****************\* 1
   |  /   4                 \
   |___________________________________________
                       Enthalphy
1-2: Compression
2-3: Heat rejection to water in gas cooler
3-4: Expansion
5-6: Evaporation

h           Enthalpy [kJ/kg]
s           Entropy [kJ/kgK]
v           Spesific volume [m3/kg]
V           Volume flow [m3/s]
t/T         Temperature [deg C]
Q           Gas quality (0-1)
%}

for i=sim_range
    
    %%%%%%%%%%%%%%%% Part with exhaust air as heat source  %%%%%%%%%%%%%%%%
    finished=false;
    dt_old=(T_ahe(i)-t_min);
    dt=dt_old*.5;
    T_disch_air(i)=T_ahe(i)-dt;
    temp_dem=Q_dem(i);
    
    while ~finished

        dt_old=dt;
        
        [Q_gc(i), Q_evap(i), T_disch_co2(i), T_evap(i), W_compr(i), V_compr(i)]= ...
            thermodynamics(T_ahe(i), T_cw(i), V_ea(i), T_disch_air(i));
        
        COP(i)=Q_gc(i)/(W_ea_fan(i)+W_compr(i));
        
        T_disch_air_old=T_disch_air(i);
        
        if V_compr(i)<V_compr_max*V_compr_min
            V_min_lim(i)=true;
        else
            V_min_lim(i)=false;
        end

        if V_compr(i) < V_compr_max*V_compr_min && ((T_disch_co2(i)-kelvin)<t_disch_max)
            dt=dt*.5;
            T_disch_air(i)=T_disch_air(i)-dt;
            if Q_gc(i)>Q_dem(i)
                V_min_lim(i)=true;
            end
        elseif T_disch_co2(i)-kelvin>t_disch_max || V_compr(i)>V_compr_max || (Q_gc(i)>Q_dem(i) && ~V_min_lim(i))
            dt=dt*.5;
            T_disch_air(i)=T_disch_air(i)+dt;
            if Q_gc(i)<Q_dem(i)
                V_min_lim(i)=false;
            end
        else
            dt=dt*.5;
            T_disch_air(i)=T_disch_air(i)-dt;
        end


        if dt_old-dt < dt_min
            finished=true;
        end
        
        if T_disch_air(i)>=T_ahe(i)    % No heat recovery due to small heat demand.
            COP(i)=0;
            Q_gc(i)=0;
            Q_evap(i)=0;
            T_disch_co2(i)=273.15;
            T_evap(i)=-99;
            W_compr(i)=0;
            V_compr(i)=0;
            finished=true;
            break
        end
        
        if finished
            T_disch_air(i)=T_disch_air_old;
        end
    end

        if Q_gc(i) > temp_dem
            corr(i)=temp_dem/(Q_gc(i));
            Q_gc(i)=temp_dem;
            Q_evap(i)=Q_evap(i)*corr(i);
            W_compr(i)=W_compr(i)*corr(i);
            Q_dem(i)=temp_dem;
            W_ea_fan(i)=W_ea_fan(i)*corr(i);            
        end
    
    % Minimum possible V_compr lower than V_compr_min --> HP turnes off
    if V_compr(i)<V_compr_max*V_compr_min
        hp_turned_off(i)=true;
        COP(i)=0;
        Q_gc(i)=0;
        Q_evap(i)=0;
        T_disch_co2(i)=273.15;      % Should not be included in graphs
        T_evap(i)=-99;              % Should not be included in graphs
        W_compr(i)=0;
        V_compr(i)=0;
        T_disch_air(i)=T_ahe(i);
        W_ea_fan(i)=0;
        corr(i)=0;
    end
    
    %%%%%%%%%%%%% End of part with exhaust air as heat source %%%%%%%%%%%%%
    
    %%%%%%%%%%%% Start of part with OA and EA as heat source %%%%%%%%%%%%
    if activate_oa && activate_ea
        T_ahe_oa(i)=(T_oa_new(i)*v_oa+T_ahe(i)*V_ea(i))/(v_oa+V_ea(i));  % Temperature is weighted average
        V_ea_oa(i)=v_oa+V_ea(i);             % Air flow is sum of oa and ea
        finished_oa=false;
        dt_oa_old=(T_ahe_oa(i)-t_min);
        dt_oa=dt_oa_old*.5;
        T_disch_air_oa(i)=T_ahe(i)-dt_oa;
        
        while ~finished_oa
            
            dt_oa_old=dt_oa;
            
            [Q_gc_oa(i), Q_evap_oa(i), T_disch_co2_oa(i), T_evap_oa(i), W_compr_oa(i), V_compr_oa(i)]= ...
                thermodynamics(T_ahe(i), T_cw(i), V_ea_oa(i), T_disch_air_oa(i));
            
            COP_oa(i)=Q_gc_oa(i)/(W_oa_fan(i)+W_ea_fan(i)+W_compr_oa(i));
            
            T_disch_air_oa_old=T_disch_air_oa(i);
            
            if V_compr_oa(i)<V_compr_max*V_compr_min
                V_min_lim_oa(i)=true;
            else
                V_min_lim_oa(i)=false;
            end
            
            if V_compr_oa(i) < V_compr_max*V_compr_min && ((T_disch_co2_oa(i)-kelvin)<t_disch_max)
                dt_oa=dt_oa*.5;
                T_disch_air_oa(i)=T_disch_air_oa(i)-dt_oa;
                if Q_gc_oa(i)>Q_dem(i)
                    V_min_lim_oa(i)=true;
                end
            elseif T_disch_co2_oa(i)-kelvin>t_disch_max || V_compr_oa(i)>V_compr_max || (Q_gc_oa(i)>Q_dem(i) && ~V_min_lim_oa(i))
                dt_oa=dt_oa*.5;
                T_disch_air_oa(i)=T_disch_air_oa(i)+dt_oa;
                if Q_gc_oa(i)<Q_dem(i)
                    V_min_lim_oa(i)=false;
                end
            else
                dt_oa=dt_oa*.5;
                T_disch_air_oa(i)=T_disch_air_oa(i)-dt_oa;
            end
            
            if dt_oa_old-dt_oa < dt_min
                finished_oa=true;
            end
            
            if T_disch_air_oa(i)>=T_ahe_oa(i)    % No heat recovery due to small demand or low outdoor temp
                COP_oa(i)=0;
                Q_gc_oa(i)=0;
                Q_evap_oa(i)=0;
                T_disch_co2_oa(i)=273.15;
                T_evap_oa(i)=-99;
                W_compr_oa(i)=0;
                V_compr_oa(i)=0;
                W_oa_fan(i)=0;
                T_disch_air_oa(i)=T_ahe_oa(i);
                finished_oa=true;
                hp_turned_off_oa(i)=true;
                break
            end
            
        end

            if Q_gc_oa(i) > temp_dem
                corr_oa(i)=temp_dem/(Q_gc_oa(i));
                Q_gc_oa(i)=temp_dem;
                Q_evap_oa(i)=Q_evap_oa(i)*corr_oa(i);
                W_compr_oa(i)=W_compr_oa(i)*corr_oa(i);
                W_oa_fan(i)=W_oa_fan(i)*corr_oa(i);
                Q_dem(i)=temp_dem;
            end
        
        % Minimum possible V_compr lower than V_compr_min --> HP turnes off
        if V_compr_oa(i)<V_compr_max*V_compr_min
            hp_turned_off_oa(i)=true;
            COP_oa(i)=0;
            Q_gc_oa(i)=0;
            Q_evap_oa(i)=0;
            T_disch_co2_oa(i)=273.15;      % Should not be included in graphs
            T_evap_oa(i)=-99;              % Should not be included in graphs
            W_compr_oa(i)=0;
            W_oa_fan(i)=0;
            V_compr_oa(i)=0;
            T_disch_air_oa(i)=T_ahe(i);
            corr_oa(i)=0;
        end
    end
    %%%%%%%% End of part with outdoor air as additional heat source %%%%%%%
    
    % Comparing numbers with and without exhaust air in order to find the
    % solution with a minimum amount of delivered energy:
    if ((W_compr_oa(i)+W_oa_fan(i)+Q_dem(i)-Q_gc_oa(i) < ...
            W_compr(i)+Q_dem(i)-Q_gc(i)) && ~oa_only)
        COP_r(i)=COP_oa(i);
        Q_gc_r(i)=Q_gc_oa(i);
        Q_evap_r(i)=Q_evap_oa(i);
        T_disch_co2_r(i)=T_disch_co2_oa(i);
        T_disch_air_r(i)=T_disch_air_oa(i);
        T_evap_r(i)=T_evap_oa(i);
        W_compr_r(i)=W_compr_oa(i);
        V_compr_r(i)=V_compr_oa(i);
        oa_used(i)=true;                % Used to indicate time steps benefiting from oa
        corr_r(i)=corr_oa(i);
        hp_turned_off_r(i)=hp_turned_off_oa(i);
    else
        COP_r(i)=COP(i);
        Q_gc_r(i)=Q_gc(i);
        Q_evap_r(i)=Q_evap(i);
        T_disch_co2_r(i)=T_disch_co2(i);
        T_disch_air_r(i)=T_disch_air(i);
        T_evap_r(i)=T_evap(i);
        W_compr_r(i)=W_compr(i);
        W_oa_fan(i)=0;
        V_compr_r(i)=V_compr(i);
        corr_r(i)=corr(i);
        hp_turned_off_r(i)=hp_turned_off(i);
    end
    
    % When operated with outdoor air as only heat source, the COP might
    % drop below 1 due to fan energy. In these cases, the hp is turned off:
    if COP_r(i) < 1
        COP_r(i)=0;
        Q_gc_r(i)=0;
        Q_evap_r(i)=0;
        T_disch_co2(i)=273.15;       % Should not be included in graphs
        T_evap_r(i)=-99;             % Should not be included in graphs
        W_compr_r(i)=0;
        W_ea_fan(i)=0;
        W_oa_fan(i)=0;
        V_compr_r(i)=0;
        T_disch_air_r(i)=T_ahe(i);
        hp_turned_off_r(i)=true;
    end
    
    if Q_gc_r(i)==0
        corr_r(i)=0;
        V_compr_r(i)=0;
        hp_turned_off_r(i)=true;
    end
    
    % Heat recovery calculations according to NS3031
    % Heat can only be recovered if the ea temp is higher than oa
    if T_ea(i) > T_oa(i)
        
        % Amount of energy possible to recover
        Q_rec_possible(i)=V_ea(i)*hcap_air*(T_ea(i)-T_oa(i));
        
        % Amount of energy recovered by the IDA ICE heat wheel
        Q_rec_hex(i)=V_ea(i)*hcap_air*(T_ea(i)-T_ahe(i));
        
        % Start of heat pump heat recovery calculation
        if T_disch_air_r(i) < T_oa(i)
            Q_rec_hp(i)=V_ea(i)*hcap_air*(T_ahe(i)-T_oa(i))*corr_r(i);
            Q_rec_possible_hp(i)=V_ea(i)*hcap_air*(T_ahe(i)-T_oa(i));
        else
            Q_rec_hp(i)=V_ea(i)*hcap_air*(T_ahe(i)-T_disch_air_r(i))*corr_r(i);
            Q_rec_possible_hp(i)=V_ea(i)*hcap_air*(T_ahe(i)-T_oa(i));
        end
        
        W_compr_rec(i)=Q_rec_hp(i)/(COP_r(i)-1)*corr_r(i);
    end

    clc
    progress=floor((i+1)*100/8760);
    fprintf('Please wait. Calculating (%i %%). %i of %i.\n',progress, i, n');
    %end
end

if oa_only
    W_oa_fan=W_ea_fan;
    W_ea_fan(1:8760,1)=0;
end

% Yearly energy calculations
Q_dem_sum=sum(Q_dem(sim_range))/1000;            % Yearly heat demand [kWh]
Q_gc_sum=sum(Q_gc_r(sim_range))/1000;            % Yearly energy from GC [kWh]
W_compr_sum=sum(W_compr_r(sim_range))/1000;      % Yearly energy consumption compr [kWh]
W_ea_fan_sum=sum(W_ea_fan(sim_range))/1000;      % Yearly increased fan energy due to evaporator [kWh]
W_oa_fan_sum=sum(W_oa_fan(sim_range))/1000;      % Yearly energy consumption from oa fan [kWh]
Q_evap_sum=sum(Q_evap_r(sim_range))/1000;        % Yearly energy from evaporator [kWh]
E_cov=Q_gc_sum/Q_dem_sum;                        % Yearly heat pump coverage [-]
Q_del=Q_dem_sum-Q_gc_sum+W_compr_sum+ ...
    W_oa_fan_sum+W_ea_fan_sum;                   % Yearly deilivered energy to building [kWh]
SPF=Q_gc_sum/ ...
    (W_compr_sum+W_ea_fan_sum+W_oa_fan_sum);     % SPF heat pump [-]
SPF_heating_syst=Q_dem_sum/Q_del;                % SPF for heating system [-]

% Yearly heat recovery calculations
eta_rec_hp=sum(Q_rec_hp)/sum(Q_rec_possible_hp);    % Heat pump recovery efficiency [-]
eta_rec_hp_real=sum(Q_rec_hp)/...
    (sum(W_compr_rec)+sum(Q_rec_possible_hp));   % Real heat pump recovery efficiency [-]
eta_rec_hybrid=(sum(Q_rec_hp)+sum(Q_rec_hex))/...
    sum(Q_rec_possible);                         % Heat rec. eff. for hp and h.exch [-]
eta_rec_hybrid_real=(sum(Q_rec_hp)+sum(Q_rec_hex))/...
    (sum(Q_rec_possible)+sum(W_compr_rec));      % Real hybrid efficiency [-]

if oa_only
    clear eta_rec*
end

save(fn);         % Saving full workspace (name defined in parameters.m)
total_time=toc(total_timer)/60;
clc
load train.mat;   % Loading sound
sound(y);         % Playing sound to inform that the simulation is finished
clear_var;        % Deleting less interesting variables from workspace
output(1,1)=Q_del;
output(2,1)=Q_gc_sum;
output(3,1)=Q_evap_sum;
output(4,1)=W_compr_sum;
output(5,1)=SPF;
output(6,1)=SPF_heating_syst;
output(7,1)=E_cov;
output(8,1)=Q_dem_sum;
save(fn_m);       % Saving reduced workspace (name defined in parameters.m)
fprintf('Simulation finished!\n')